from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("fluent_pages", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="BlogPage",
            fields=[
                (
                    "urlnode_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        on_delete=models.CASCADE,
                        primary_key=True,
                        serialize=False,
                        to="fluent_pages.UrlNode",
                    ),
                ),
            ],
            options={
                "db_table": "pagetype_blogpage_blogpage",
                "verbose_name": "Blog module",
                "verbose_name_plural": "Blog modules",
            },
            bases=("fluent_pages.htmlpage",),
        ),
    ]



from os.path import dirname, join

from example_standalone.settings import *

INSTALLED_APPS += (
    # Add fluent pages with a simple page type:
    "fluent_pages",
    #'fluent_pages.pagetypes.fluentpage',
    "fluent_pages.pagetypes.flatpage",
    # Add the page type for adding the "blogs" root to the page tree.
    "fluent_blogs.pagetypes.blogpage",
    # fluent-pages dependencies:
    "mptt",
    "polymorphic",
    "polymorphic_tree",
)

ROOT_URLCONF = "example_fluent_pages.urls"

TEMPLATES[0]["DIRS"] = (join(dirname(__file__), "templates"),)



# Django settings for example project.
# Add parent path,
# Allow starting the app without installing the module.
import sys
from os.path import dirname, join, realpath

sys.path.insert(0, dirname(dirname(dirname(realpath(__file__)))))

DEBUG = True

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": dirname(dirname(__file__)) + "/demo.db",
    }
}

TIME_ZONE = "Europe/Amsterdam"
LANGUAGE_CODE = "en"  # Important when switching to multilingual support!
SITE_ID = 1

USE_I18N = True
USE_L10N = True
USE_TZ = True

MEDIA_ROOT = join(dirname(__file__), "media")
MEDIA_URL = "/media/"
STATIC_ROOT = join(dirname(__file__), "static")
STATIC_URL = "/static/"

STATICFILES_DIRS = ()
STATICFILES_FINDERS = (
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
    #    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = "-#@bi6bue%#1j)6+4b&#i0g-*xro@%f@_#zwv=2-g_@n3n_kj5"

MIDDLEWARE = (
    "django.middleware.common.CommonMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
)

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [
            join(dirname(__file__), "templates"),
        ],
        "OPTIONS": {
            "loaders": (
                "django.template.loaders.filesystem.Loader",
                "django.template.loaders.app_directories.Loader",
            ),
            "context_processors": (
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.request",
                "django.template.context_processors.static",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ),
        },
    },
]

ROOT_URLCONF = "example_standalone.urls"

INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.admin",
    "django.contrib.admindocs",
    # Site theme.
    # Placed above all other apps, to override third party apps
    "theme1",
    # Blog module
    "fluent_blogs",
    # Contents plugins
    "fluent_contents",
    "fluent_contents.plugins.code",
    "fluent_contents.plugins.commentsarea",
    #'fluent_contents.plugins.disquswidgets',
    #'fluent_contents.plugins.formdesignerlink',
    "fluent_contents.plugins.gist",
    "fluent_contents.plugins.googledocsviewer",
    "fluent_contents.plugins.iframe",
    "fluent_contents.plugins.markup",
    "fluent_contents.plugins.oembeditem",
    "fluent_contents.plugins.rawhtml",
    #'fluent_contents.plugins.sharedcontent',
    "fluent_contents.plugins.text",
    #'fluent_contents.plugins.twitterfeed',
    # Other (optional) dependencies
    "categories_i18n",  # default 'categories' app, can be changed.
    "django_comments",  # should be below theme1
    "django_wysiwyg",  # for 'text' plugin
    #'disqus',                       # for 'disqus' plugin
    #'form_designer',                # for 'formdesignerlink' plugin
    "parler",  # provides the multilingual support.
    "slug_preview",  # nicer slugs in the admin.
    "taggit",  # optional tagging support.
    "taggit_autosuggest",  # optional autocompletion support for tags
    "tinymce",  # Used by 'text' plugin, see DJANGO_WYSIWYG_FLAVOR
)

TEST_RUNNER = "django.test.runner.DiscoverRunner"  # silence system checks

DJANGO_WYSIWYG_FLAVOR = "tinymce_advanced"

# Being explicit for now
FLUENT_BLOGS_BASE_TEMPLATE = "fluent_blogs/base.html"


LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "filters": {
        "require_debug_false": {"()": "django.utils.log.RequireDebugFalse"},
    },
    "handlers": {
        "mail_admins": {
            "level": "ERROR",
            "filters": ["require_debug_false"],
            "class": "django.utils.log.AdminEmailHandler",
        }
    },
    "loggers": {
        "django.request": {
            "handlers": ["mail_admins"],
            "level": "ERROR",
            "propagate": True,
        },
    },
}

# DISQUS_API_KEY = ''
# DISQUS_WEBSITE_SHORTNAME = ''



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("fluent_blogs", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="entry_translation",
            name="intro",
            field=models.TextField(null=True, verbose_name="Introtext"),
        ),
    ]



from django.conf import settings
from django.urls import path, re_path

from fluent_blogs import appsettings
from fluent_blogs.views.entries import (
    EntryArchiveIndex,
    EntryAuthorArchive,
    EntryCategoryArchive,
    EntryDayArchive,
    EntryDetail,
    EntryMonthArchive,
    EntryShortLink,
    EntryTagArchive,
    EntryYearArchive,
)
from fluent_blogs.views.feeds import (
    LatestAuthorEntriesFeed,
    LatestCategoryEntriesFeed,
    LatestEntriesFeed,
    LatestTagEntriesFeed,
)


def _get_entry_regex():
    # Configurable permalink style!
    regex = (
        appsettings.FLUENT_BLOGS_ENTRY_LINK_STYLE.replace("{year}", r"(?P<year>\d{4})")
        .replace("{month}", r"(?P<month>\d{2})")
        .replace("{day}", r"(?P<day>\d{2})")
        .replace("{slug}", r"(?P<slug>[-\w]+)")
        .replace("{pk}", r"(?P<pk>[-\w]+)")
    )
    return "^{}$".format(regex.lstrip("/"))


urlpatterns = [
    # Index
    path("", EntryArchiveIndex.as_view(), name="entry_archive_index"),
    path("page/<int:page>/", EntryArchiveIndex.as_view(), name="entry_archive_index_paginated"),
    re_path(
        r"^feed.rss2$", LatestEntriesFeed.as_view(format="rss2.0"), name="entry_archive_index_rss"
    ),
    re_path(
        r"^feed.atom$", LatestEntriesFeed.as_view(format="atom1"), name="entry_archive_index_atom"
    ),
    # Archives
    re_path(r"^(?P<year>\d{4})/$", EntryYearArchive.as_view(), name="entry_archive_year"),
    re_path(
        r"^(?P<year>\d{4})/(?P<month>\d{2})/$",
        EntryMonthArchive.as_view(),
        name="entry_archive_month",
    ),
    re_path(
        r"^(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d{2})/$",
        EntryDayArchive.as_view(),
        name="entry_archive_day",
    ),
    # Categories
    re_path(
        r"^categories/(?P<slug>[-\w]+)/$",
        EntryCategoryArchive.as_view(),
        name="entry_archive_category",
    ),
    re_path(
        r"^categories/(?P<slug>[-\w]+)/page/(?P<page>\d+)/$",
        EntryCategoryArchive.as_view(),
        name="entry_archive_category_paginated",
    ),
    re_path(
        r"^categories/(?P<slug>[-\w]+)/feed.rss2$",
        LatestCategoryEntriesFeed.as_view(format="rss2.0"),
        name="entry_archive_category_rss",
    ),
    re_path(
        r"^categories/(?P<slug>[-\w]+)/feed.atom$",
        LatestCategoryEntriesFeed.as_view(format="atom1"),
        name="entry_archive_category_atom",
    ),
    # Authors
    re_path(
        r"^authors/(?P<slug>[-_@.\w]+)/$",
        EntryAuthorArchive.as_view(),
        name="entry_archive_author",
    ),
    re_path(
        r"^authors/(?P<slug>[-_@.\w]+)/page/(?P<page>\d+)/$",
        EntryAuthorArchive.as_view(),
        name="entry_archive_author_paginated",
    ),
    re_path(
        r"^authors/(?P<slug>[-_@.\w]+)/feed.rss2$",
        LatestAuthorEntriesFeed.as_view(format="rss2.0"),
        name="entry_archive_author_rss",
    ),
    re_path(
        r"^authors/(?P<slug>[-_@.\w]+)/feed.atom$",
        LatestAuthorEntriesFeed.as_view(format="atom1"),
        name="entry_archive_author_atom",
    ),
    # Short link
    path(
        "<int:pk>/", EntryShortLink.as_view(), name="entry_shortlink"
    ),  # Short link can also be used as GUID.
    # Entries
    re_path(_get_entry_regex(), EntryDetail.as_view(), name="entry_detail"),
]


if "taggit" in settings.INSTALLED_APPS:
    urlpatterns += [
        re_path(r"^tags/(?P<slug>[-\w]+)/$", EntryTagArchive.as_view(), name="entry_archive_tag"),
        re_path(
            r"^tags/(?P<slug>[-\w]+)/page/(?P<page>\d+)/$",
            EntryTagArchive.as_view(),
            name="entry_archive_tag_paginated",
        ),
        re_path(
            r"^tags/(?P<slug>[-\w]+)/feed.rss2$",
            LatestTagEntriesFeed.as_view(format="rss2.0"),
            name="entry_archive_tag_rss",
        ),
        re_path(
            r"^tags/(?P<slug>[-\w]+)/feed.atom$",
            LatestTagEntriesFeed.as_view(format="atom1"),
            name="entry_archive_tag_atom",
        ),
    ]



from django.conf import settings


def multilingual(request):
    lang_list = settings.PARLER_LANGUAGES.get(settings.SITE_ID, ())
    return {
        "MULTILINGUAL_LANGUAGES": [item["code"] for item in lang_list],
    }



from django.contrib import admin
from django.urls import include, path

admin.autodiscover()

urlpatterns = [
    path("admin/apps/tinymce/", include("tinymce.urls")),
    path("admin/apps/tags/", include("taggit_autosuggest.urls")),
    path("admin/", admin.site.urls),
    path("comments/", include("fluent_comments.urls")),
    # url(r'^forms/', include('form_designer.urls')),
    path("", include("fluent_blogs.urls")),
]



from os.path import dirname, join

from example_standalone.settings import *

PARLER_DEFAULT_LANGUAGE_CODE = "en"  # defaults to LANGUAGE_CODE

TEMPLATES[0]["OPTIONS"]["context_processors"] += (
    "example_multilingual.context_processors.multilingual",
)

SITE_ID = 1

PARLER_LANGUAGES = {
    # site ID 1:
    1: (
        {"code": "en"},
        {"code": "fr"},
        {"code": "de"},
        {"code": "es"},
        {"code": "nl"},
    ),
    "default": {
        "hide_untranslated": False,  # being explicit here.
        "fallback": "en",  # defaults to PARLER_DEFAULT_LANGUAGE_CODE or LANGUAGE_CODE
    },
}

# NOTE: this middleware is not required, you can also use our own variation to set the frontend language.
MIDDLEWARE += ("django.middleware.locale.LocaleMiddleware",)

ROOT_URLCONF = "example_multilingual.urls"

TEMPLATES[0]["DIRS"] = (join(dirname(__file__), "templates"),)



from example_standalone.settings import *

INSTALLED_APPS = (
    "fluent_comments",
    "crispy_forms",
) + INSTALLED_APPS

COMMENTS_APP = "fluent_comments"

ROOT_URLCONF = "example_fluent_comments.urls"

# AKISMET_API_KEY = ''

# Being explicit here, auto includes comment-static files:
FLUENT_BLOGS_INCLUDE_STATIC_FILES = True



import slug_preview.models
from django.conf import settings
from django.db import migrations, models

import fluent_blogs.appsettings
import fluent_blogs.base_models


def make_site(apps, schema_editor):
    Site = apps.get_model("sites", "Site")
    if Site.objects.count() == 0:
        site = Site()
        site.pk = settings.SITE_ID
        site.name = "example"
        site.domain = "example.com"
        site.save()


def remove_site(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [
        ("sites", "0001_initial"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        migrations.swappable_dependency(fluent_blogs.appsettings.FLUENT_BLOGS_CATEGORY_MODEL),
    ]

    operations = [
        migrations.RunPython(make_site, reverse_code=remove_site),
        migrations.CreateModel(
            name="Entry",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID", serialize=False, auto_created=True, primary_key=True
                    ),
                ),
                (
                    "enable_comments",
                    models.BooleanField(default=True, verbose_name="Enable comments"),
                ),
                (
                    "status",
                    models.CharField(
                        default="d",
                        max_length=1,
                        verbose_name="status",
                        db_index=True,
                        choices=[("p", "Published"), ("d", "Draft")],
                    ),
                ),
                (
                    "publication_date",
                    models.DateTimeField(
                        help_text='When the entry should go live, status must be "Published".',
                        null=True,
                        verbose_name="publication date",
                        db_index=True,
                    ),
                ),
                (
                    "publication_end_date",
                    models.DateTimeField(
                        db_index=True, null=True, verbose_name="publication end date", blank=True
                    ),
                ),
                (
                    "creation_date",
                    models.DateTimeField(auto_now_add=True, verbose_name="creation date"),
                ),
                (
                    "modification_date",
                    models.DateTimeField(auto_now=True, verbose_name="last modification"),
                ),
                (
                    "author",
                    models.ForeignKey(
                        verbose_name="author",
                        on_delete=models.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "categories",
                    models.ManyToManyField(
                        to=fluent_blogs.appsettings.FLUENT_BLOGS_CATEGORY_MODEL,
                        verbose_name="Categories",
                        blank=True,
                    ),
                ),
                (
                    "parent_site",
                    models.ForeignKey(
                        default=fluent_blogs.base_models._get_current_site,
                        on_delete=models.CASCADE,
                        editable=False,
                        to="sites.Site",
                    ),
                ),
            ],
            options={
                "ordering": ("-publication_date",),
                "verbose_name": "Blog entry",
                "verbose_name_plural": "Blog entries",
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="Entry_Translation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID", serialize=False, auto_created=True, primary_key=True
                    ),
                ),
                (
                    "language_code",
                    models.CharField(db_index=True, max_length=15, verbose_name="Language"),
                ),
                ("title", models.CharField(max_length=200, verbose_name="Title")),
                ("slug", slug_preview.models.SlugPreviewField(verbose_name="Slug")),
                ("intro", models.TextField(verbose_name="Introtext")),
                (
                    "meta_keywords",
                    models.CharField(
                        default="",
                        help_text="When this field is not filled in, the the tags will be used.",
                        max_length=255,
                        verbose_name="keywords",
                        blank=True,
                    ),
                ),
                (
                    "meta_description",
                    models.CharField(
                        default="",
                        help_text="When this field is not filled in, the contents or intro text will be used.",
                        max_length=255,
                        verbose_name="description",
                        blank=True,
                    ),
                ),
                (
                    "meta_title",
                    models.CharField(
                        help_text="When this field is not filled in, the menu title text will be used.",
                        max_length=255,
                        null=True,
                        verbose_name="page title",
                        blank=True,
                    ),
                ),
                (
                    "master",
                    models.ForeignKey(
                        related_name="translations",
                        on_delete=models.CASCADE,
                        editable=False,
                        to="fluent_blogs.Entry",
                        null=True,
                    ),
                ),
            ],
            options={
                "verbose_name": "Blog entry translation",
                "verbose_name_plural": "Blog entry translations",
            },
            bases=(models.Model,),
        ),
    ]



from django.conf import settings
from django.contrib import admin
from django.contrib.admin.widgets import AdminTextareaWidget, AdminTextInputWidget
from django.core.exceptions import ImproperlyConfigured
from django.urls import NoReverseMatch
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.timezone import now
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _
from fluent_contents.admin import PlaceholderFieldAdmin
from fluent_utils.dry.admin import MultiSiteAdminMixin
from parler.admin import TranslatableAdmin
from parler.models import TranslationDoesNotExist

from fluent_blogs import appsettings
from fluent_blogs.admin.forms import (
    AbstractEntryBaseAdminForm,
    AbstractTranslatableEntryBaseAdminForm,
)
from fluent_blogs.base_models import AbstractEntryBase
from fluent_blogs.models import get_entry_model

EntryModel = get_entry_model()


class AbstractEntryBaseAdmin(MultiSiteAdminMixin, PlaceholderFieldAdmin):
    """
    The base functionality of the admin, which only uses the fields of the
    :class:`~fluent_blogs.base_models.AbstractEntryBase` model.
    Everything else is branched off in the :class:`EntryAdmin` class.
    """

    filter_site = appsettings.FLUENT_BLOGS_FILTER_SITE_ID
    list_display = ("title", "status_column", "modification_date", "actions_column")
    list_filter = ("status",)
    date_hierarchy = "publication_date"
    search_fields = ("slug", "title")
    actions = ["make_published"]
    form = AbstractEntryBaseAdminForm
    prepopulated_fields = {
        "slug": ("title",),
    }
    radio_fields = {
        "status": admin.HORIZONTAL,
    }
    raw_id_fields = ("author",)

    FIELDSET_GENERAL = (
        None,
        {
            "fields": (
                "title",
                "slug",
                "status",
            ),
        },
    )
    FIELDSET_PUBLICATION = (
        _("Publication settings"),
        {
            "fields": ("publication_date", "publication_end_date", "author"),
            "classes": ("collapse",),
        },
    )

    class Media:
        css = {"all": ("fluent_blogs/admin/admin.css",)}

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = list(super().get_readonly_fields(request, obj))
        if not request.user.is_superuser:
            readonly_fields.append("author")
        return readonly_fields

    def save_model(self, request, obj, form, change):
        # Automatically store the user in the author field.
        if not obj.author_id:
            obj.author = request.user

        if not obj.publication_date:
            # auto_now_add makes the field uneditable.
            # default fills the field before the post is written (too early)
            obj.publication_date = now()
        obj.save()

    def formfield_for_dbfield(self, db_field, **kwargs):
        """
        Allow formfield_overrides to contain field names too.
        """
        overrides = self.formfield_overrides.get(db_field.name)
        if overrides:
            kwargs.update(overrides)

        field = super().formfield_for_dbfield(db_field, **kwargs)

        # Pass user to the form.
        if db_field.name == "author":
            field.user = kwargs["request"].user
        return field

    def render_change_form(self, request, context, add=False, change=False, form_url="", obj=None):
        # When the page is accessed via a pagetype, warn that the node can't be previewed yet.
        context["preview_error"] = ""
        if "fluent_pages" in settings.INSTALLED_APPS:
            from fluent_pages.urlresolvers import MultipleReverseMatch, PageTypeNotMounted

            try:
                self._reverse_blogpage_index(request, obj)
            except PageTypeNotMounted:
                from fluent_blogs.pagetypes.blogpage.models import BlogPage

                context["preview_error"] = gettext(
                    "The blog entry can't be previewed yet, a '{page_type_name}' page needs to be created first."
                ).format(page_type_name=BlogPage._meta.verbose_name)
            except MultipleReverseMatch:
                # When 'entry_archive_index is ambiguous (because there are multiple blog nodes in the fluent-pages tree),
                # the edit page will automatically pick an option.
                pass
            except NoReverseMatch:
                # Since forgetting the pagetype app is easy, give off a warning to help developers
                # find their way with these apps.
                raise ImproperlyConfigured(
                    "To use django-fluent-blogs, either include('fluent_blogs.urls') in the URLConf, "
                    "or add the 'fluent_blogs.pagetypes.blogpage' app to the INSTALLED_APPS."
                )

        return super().render_change_form(request, context, add, change, form_url, obj)

    def _reverse_blogpage_index(self, request, obj=None):
        # Internal method with "protected access" to handle translation differences.
        # This is only called when 'fluent_pages' is in the INSTALLED_APPS.
        from fluent_pages.urlresolvers import mixed_reverse

        return mixed_reverse("entry_archive_index")

    # ---- List code ----

    STATUS_ICONS = {
        AbstractEntryBase.PUBLISHED: "admin/img/icon-yes.svg",
        AbstractEntryBase.HIDDEN: "admin/img/icon-alert.svg",
        AbstractEntryBase.DRAFT: "admin/img/icon-unknown.svg",
    }

    @classmethod
    def get_status_column(cls, entry):
        # Create a status column, is also reused by templatetags/fluent_blogs_admin_tags.py
        try:
            icon = cls.STATUS_ICONS[entry.status]
        except KeyError:
            return ""
        title = next(rec[1] for rec in AbstractEntryBase.STATUSES if rec[0] == entry.status)
        return format_html(
            '<img src="{static_url}{icon}" alt="{title}" title="{title}" />',
            static_url=settings.STATIC_URL,
            icon=icon,
            title=title,
        )

    def status_column(self, entry):
        # Method is needed because can't assign attributes to a class method
        return self.get_status_column(entry)

    status_column.allow_tags = True
    status_column.short_description = _("Status")

    @classmethod
    def get_actions_column(cls, entry):
        return mark_safe(" ".join(cls._actions_column_icons(entry)))

    @classmethod
    def _actions_column_icons(cls, entry):
        actions = []
        if cls.can_preview_object(entry):
            try:
                url = entry.get_absolute_url()
            except (NoReverseMatch, TranslationDoesNotExist):
                # A Blog Entry is already added, but the URL can no longer be resolved.
                # This can either mean that urls.py is missing a 'fluent_blogs.urls' (unlikely),
                # or that this is a PageTypeNotMounted exception because the "Blog page" node was removed.
                # In the second case, the edit page should still be reachable, and the "view on site" link will give an alert.
                pass
            else:
                actions.append(
                    format_html(
                        '<a href="{url}" title="{title}" target="_blank"><img src="{static}fluent_blogs/img/admin/world.gif" width="16" height="16" alt="{title}" /></a>',
                        url=url,
                        title=_("View on site"),
                        static=settings.STATIC_URL,
                    )
                )
        return actions

    @classmethod
    def can_preview_object(cls, entry):
        """Override whether the node can be previewed."""
        return hasattr(entry, "get_absolute_url") and entry.is_published

    def actions_column(self, entry):
        return self.get_actions_column(entry)

    actions_column.allow_tags = True
    actions_column.short_description = _("Actions")

    def make_published(self, request, queryset):
        rows_updated = queryset.update(status=AbstractEntryBase.PUBLISHED)

        if rows_updated == 1:
            message = "1 entry was marked as published."
        else:
            message = f"{rows_updated} entries were marked as published."
        self.message_user(request, message)

    make_published.short_description = _("Mark selected entries as published")


class AbstractTranslatableEntryBaseAdmin(TranslatableAdmin, AbstractEntryBaseAdmin):
    """
    The base functionality of the admin, which only uses the fields of the
    :class:`~fluent_blogs.base_models.AbstractTranslatedEntryBase` model.
    Everything else is branched off in the :class:`EntryAdmin` class.
    """

    form = AbstractTranslatableEntryBaseAdminForm
    list_display = (
        "title",
        "language_column",
        "status_column",
        "modification_date",
        "actions_column",
    )
    list_filter = ["status"]
    if getattr(settings, "PARLER_LANGUAGES", None):
        list_filter.append("translations__language_code")
    search_fields = ("translations__slug", "translations__title")
    prepopulated_fields = (
        {}
    )  # Not supported by django-parler 0.9.2, using get_prepopulated_fields() as workaround.
    change_form_template = (
        None  # Avoid the django-parler template override trick, we do this ourselves.
    )

    def get_prepopulated_fields(self, request, obj=None):
        # Still allow to override self.prepopulated_fields in other custom classes,
        # but default to the settings which are compatible with django-parler.
        return self.prepopulated_fields or {
            "slug": ("title",),
        }

    def _reverse_blogpage_index(self, request, obj=None):
        # Updated mixed_reverse() call, with language code included.
        from fluent_pages.urlresolvers import mixed_reverse

        language_code = self.get_form_language(request, obj)
        return mixed_reverse("entry_archive_index", language_code=language_code)

    def get_language_short_title(self, language_code):
        """
        Turn the language code to uppercase.
        """
        return language_code.upper()

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context["FLUENT_BLOGS_IS_TRANSLATABLE"] = True
        return super().changelist_view(request, extra_context)


class SeoEntryAdminMixin:
    """
    Mixin for the SEO fields.
    """

    FIELDSET_SEO = (
        _("SEO settings"),
        {
            "fields": (
                "meta_title",
                "meta_keywords",
                "meta_description",
            ),
            "classes": ("collapse",),
        },
    )

    # AbstractEntryBaseAdmin allows to specify the widgets by field name,
    # which formfield_overrides doesn't support by default.
    formfield_overrides = {
        "meta_keywords": {"widget": AdminTextInputWidget(attrs={"class": "vLargeTextField"})},
        "meta_description": {"widget": AdminTextareaWidget(attrs={"rows": 3})},
    }



import fluent_pages.urls
from django.conf.urls.i18n import i18n_patterns
from django.contrib import admin
from django.urls import include, path

urlpatterns = i18n_patterns(
    path("admin/", admin.site.urls),
    path("", include(fluent_pages.urls)),
)
